package org.jboss.remoting;

import org.jboss.logging.Logger;
import org.jboss.util.TimerQueue;
import org.jboss.util.TimerTask;

/**
 * Internal agent class to ping the remote server to keep lease alive.
 *
 * @author <a href="mailto:telrod@e2technologies.net">Tom Elrod</a>
 */
public class LeasePinger extends TimerTask
{
   private Client client = null;
   private long pingPeriod = -1;

   private TimerQueue pingTimer = null;

   private static final Logger log = Logger.getLogger(LeasePinger.class);

   public LeasePinger(Client remotingClient)
   {
      this.client = remotingClient;
      pingTimer = new TimerQueue("LeasePinger-" + client.getSessionId());
   }

   public void startPing(long leaseTimeout)
   {
      this.pingPeriod = leaseTimeout;
      pingTimer.start();
      pingTimer.schedule(this, pingPeriod);
   }

   public void stopPing()
   {
      pingTimer.stop();
   }

   public void execute() throws Exception
   {
      pingTimer.schedule(this, pingPeriod);

      if(client != null)
      {
         try
         {
            client.invoke("$PING$");
         }
         catch(Throwable throwable)
         {
            log.warn("Error sending lease ping to server.");
         }
      }
   }
}
