/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.remoting.transport.http.ssl;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.util.Map;

import javax.net.SocketFactory;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocketFactory;

import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.transport.http.HTTPClientInvoker;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class HTTPSClientInvoker extends HTTPClientInvoker
{
   /**
    * A property to override the default https url host verification
    */
   public static final String IGNORE_HTTPS_HOST = "org.jboss.security.ignoreHttpsHost";

   public HTTPSClientInvoker(InvokerLocator locator)
   {
      super(locator);
   }

   public HTTPSClientInvoker(InvokerLocator locator, Map configuration)
   {
      super(locator, configuration);
   }


   /**
    * Checks to see if org.jboss.security.ignoreHttpHost property is set to true, and if it
    * is, will ste the host name verifier so that will accept any host.
    *
    * @return
    * @throws IOException
    */
   protected HttpURLConnection createURLConnection(String url, Map metadata) throws IOException
   {
      HttpURLConnection conn = super.createURLConnection(url, metadata);

      boolean ignoreHTTPSHost = Boolean.getBoolean(IGNORE_HTTPS_HOST);
      String ignoreHost = (String) metadata.get(IGNORE_HTTPS_HOST);
      if(ignoreHost != null && ignoreHost.length() > 0)
      {
         ignoreHTTPSHost = Boolean.valueOf(ignoreHost).booleanValue();
      }

      if (conn instanceof HttpsURLConnection)
      {
         HttpsURLConnection sconn = (HttpsURLConnection) conn;

         SocketFactory socketFactory = getSocketFactory();
         if (socketFactory != null && socketFactory instanceof SSLSocketFactory)
         {
            sconn.setSSLSocketFactory((SSLSocketFactory) socketFactory);
         }

         if(ignoreHTTPSHost)
         {
         sconn.setHostnameVerifier(new AnyhostVerifier());
      }
      }

      return conn;
   }

   private class AnyhostVerifier implements HostnameVerifier
   {

      public boolean verify(String s, SSLSession sslSession)
      {
         return true;
      }
   }
}