/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

/*
 * Created on Aug 26, 2005
 */

package org.jboss.remoting.transport.multiplex;

import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;


/**
 * A FastPipedInputStream.

 * @author <a href="mailto:r.sigal@computer.org">Ron Sigal</a>
 * @version $Revision: 1.5 $
 * <p>
 * Copyright (c) 2005
 * </p>
 */

public class FastPipedInputStream extends PipedInputStream
{

/**
 * Create a new FastPipedInputStream.
 * 
 * 
 */
   public FastPipedInputStream()
   {
      super();

   }
   
   
/**
 * Create a new FastPipedInputStream.
 * 
 * @param src
 * @throws java.io.IOException
 */
   public FastPipedInputStream(PipedOutputStream src) throws IOException
   {
      super(src);
   }


/**
 * 
 */
   protected synchronized void receive(int b) throws IOException 
   {
      try
      {
         super.receive(b);
      }
      catch (IOException e)
      {
         String message = e.getMessage();
         if (message != null && message.equals("Read end dead"))
            receiveAnyWay(b);
         else
            throw e;
      }
     
      notifyAll();
   }
   
   
   protected synchronized void receiveAnyWay(int b) throws IOException
   {
      // If buffer is full, wait until there is free space.
      while (in == out)
      {
 	    notifyAll();
 	    
 	    try
 	    {
 	       wait(1000);
 	    }
 	    catch (InterruptedException ex)
 	    {
 	       throw new java.io.InterruptedIOException();
 	    }
      }
      
      if (in < 0)
      {
         in = 0;
         out = 0;
      }
      
      buffer[in++] = (byte)(b & 0xFF);
      
      if (in >= buffer.length)
      {
         in = 0;
      }
   }
}

