/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

/*
 * Created on Oct 15, 2005
 */
 
package org.jboss.remoting.transport.multiplex.utility;

import java.io.ByteArrayOutputStream;



/**
 * A ShrinkableByteArrayOutputStream.

 * @author <a href="mailto:r.sigal@computer.org">Ron Sigal</a>
 * @version $Revision: 1.3 $
 * <p>
 * Copyright (c) 2005
 * </p>
 */

public class ShrinkableByteArrayOutputStream extends ByteArrayOutputStream
{
   static private int MIN_LENGTH = 1024;
   private int used = 0;
   private int nextUsed = 0;
   private int bytesReturned = 0;
   
   
   /**
    * 
    * Create a new ShrinkableByteArrayOutputStream.
    * 
    *
    */
   public ShrinkableByteArrayOutputStream()
   {
      super(MIN_LENGTH);
   }
   
   
   /**
    *  FIXME Comment this
    * 
    * @return
    */
   public int available()
   {
      return count - nextUsed;
   }
   
   
   /**
    *  FIXME Comment this
    * 
    * @return
    */
   public int bytesReturned()
   {
      return bytesReturned;
   }
   
   
   /**
    *  FIXME Comment this
    * 
    * @return
    */
   public int start()
   {
      return used;
   }
   
   
   /**
    *  FIXME Comment this
    * 
    * @param length
    * @return
    */
   public synchronized byte[] toByteArray(int length)
   {
      used = nextUsed;
      int currentLength = buf.length;
      int quarterLength = currentLength >> 2;
      
      if (currentLength > MIN_LENGTH && currentLength - used <= quarterLength)
      {
         byte newbuf[] = new byte[currentLength >> 1];
         System.arraycopy(buf, used, newbuf, 0, available());
         buf = newbuf;
         count -= used;
         used = 0;
      }
      
      bytesReturned = Math.min(count - used, length);
      nextUsed = used + bytesReturned;
      return buf;
   }

   
   /**
    * Writes the specified byte to this byte array output stream. 
    *
    * @param   b   the byte to be written.
    */
   public synchronized void write(int b) 
   {
      if (count == buf.length && used > buf.length >> 1)
      {
         System.arraycopy(buf, used, buf, 0, count - used);
         nextUsed -= used;
         count -= used;
         used = 0;
      }
      
      super.write(b);
   }
   
   
   /**
    * Writes <code>len</code> bytes from the specified byte array 
    * starting at offset <code>off</code> to this byte array output stream.
    *
    * @param   b     the data.
    * @param   off   the start offset in the data.
    * @param   len   the number of bytes to write.
    */
   public synchronized void write(byte b[], int off, int len)
   {
       if (count + len > buf.length && used > buf.length >> 1)
       {
          System.arraycopy(buf, used, buf, 0, count - used);
          nextUsed -= used;
          count -= used;
          used = 0;
       }
       
       super.write(b, off, len);
   }
   
   
   protected int getCount()
   {
      return super.size();
   }
   
   
   protected int getLength()
   {
      return buf.length;
   }
   
   
   protected int getNextUsed()
   {
      return nextUsed;
   }
   
   
   protected int getUsed()
   {
      return used;
   }

}

