/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.remoting.transport.socket.ssl;

import java.io.IOException;
import java.net.Socket;
import java.util.Map;
import javax.net.SocketFactory;
import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.security.SSLSocketBuilder;
import org.jboss.remoting.transport.socket.SocketClientInvoker;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class SSLSocketClientInvoker extends SocketClientInvoker
{
   public SSLSocketClientInvoker(InvokerLocator locator) throws IOException
   {
      super(locator);
   }

   public SSLSocketClientInvoker(InvokerLocator locator, Map configuration) throws IOException
   {
      super(locator, configuration);
   }

   protected SocketFactory createSocketFactory(Map configuration)
   {
      SocketFactory sf = super.createSocketFactory(configuration);

      if (sf != null)
         return sf;

      try
      {
         SSLSocketBuilder server = new SSLSocketBuilder();
         sf = server.createSSLSocketFactory();
      }
      catch(Exception e)
      {
         log.error("Error creating SSL Socket Factory for client invoker.", e);
      }

      return sf;
   }


   protected Socket createSocket(String address, int port) throws IOException
   {
//      SocketFactory sf = null;
//      try
//      {
//         sf = getSocketFactory();
//
//         if (sf == null)
//         {
//            SSLSocketBuilder server = new SSLSocketBuilder();
//            sf = server.createSSLSocketFactory();
//         }
//      }
//      catch(Exception e)
//      {
//         log.error("Error creating SSL Socket Factory for client invoker.", e);
//         throw new IOException("Error creating SSL Socket Factory.  Root cause: " + e.getMessage());
//      }

      SocketFactory sf = getSocketFactory();

      if (sf == null)
      {
         String message = "No SocketFactory has been created";
         log.error(message);
         throw new IOException(message);
      }

      Socket s = sf.createSocket(address, port);
      return s;
   }

}